import datetime
import logging
import textwrap

import dateutil.parser
import humanize
import pytz


class EpochParser:
    @classmethod
    def parse(cls, timestamp: str):
        epochtime = float(timestamp)
        logging.warning(f"assuming {epochtime} is UTC")
        return datetime.datetime.fromtimestamp(epochtime).astimezone(pytz.utc)


class DateStampParser:
    @classmethod
    def parse(cls, timestamp: str):
        return dateutil.parser.parse(timestamp)


def main(args):
    tz = pytz.timezone("US/Pacific")
    logging.debug(args.timestamp)

    ts = " ".join(args.timestamp)
    epoch = None
    try:
        epoch = float(ts)
    except ValueError:
        ...

    logging.debug(f"{ts=}, {type(ts)=}, {epoch=}")

    dt = (
        EpochParser.parse(ts) if isinstance(epoch, float) else DateStampParser.parse(ts)
    )

    dt2 = dt.astimezone(tz)

    now = datetime.datetime.now(tz)

    meta = "from now" if now < dt else "ago"

    relative = humanize.naturaldelta(now - dt2)
    out = textwrap.dedent(
        f"""\
    input:     {args.timestamp}
    asiso:     {dt.isoformat()}
    epoch:     {dt.timestamp()}
    converted: {dt}
    mytime:    {dt2}
    relative:  {relative} {meta}\
    """
    )

    print(out)



import argparse
import logging
import sys

from comparetap import __version__

from . import main as mainmod

__author__ = "Taylor Monacelli"
__copyright__ = "Taylor Monacelli"
__license__ = "MPL-2.0"

_logger = logging.getLogger(__name__)


def parse_args(args):
    """Parse command line parameters

    Args:
      args (List[str]): command line parameters as list of strings
          (for example  ``["--help"]``).

    Returns:
      :obj:`argparse.Namespace`: command line parameters namespace
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--version",
        action="version",
        version="comparetap {ver}".format(ver=__version__),
    )
    parser.add_argument(dest="timestamp", nargs="+")
    parser.add_argument(
        "-v",
        "--verbose",
        dest="loglevel",
        help="set loglevel to INFO",
        action="store_const",
        const=logging.INFO,
    )
    parser.add_argument(
        "-vv",
        "--very-verbose",
        dest="loglevel",
        help="set loglevel to DEBUG",
        action="store_const",
        const=logging.DEBUG,
    )
    return parser.parse_args(args)


def setup_logging(loglevel):
    logformat = "[%(asctime)s] %(levelname)s:%(name)s:%(message)s"
    logging.basicConfig(
        level=loglevel, stream=sys.stdout, format=logformat, datefmt="%Y-%m-%d %H:%M:%S"
    )


def main(args):
    args = parse_args(args)
    setup_logging(args.loglevel)
    _logger.debug("Starting crazy calculations...")
    mainmod.main(args)
    _logger.info("Script ends here")


def run():
    """Calls :func:`main` passing the CLI arguments extracted from :obj:`sys.argv`

    This function can be used as entry point to create console scripts with setuptools.
    """
    main(sys.argv[1:])


if __name__ == "__main__":
    # ^  This is a guard statement that will prevent the following code from
    #    being executed in the case someone imports this file instead of
    #    executing it as a script.
    #    https://docs.python.org/3/library/__main__.html

    # After installing your project with pip, users can also run your Python
    # modules as scripts via the ``-m`` flag, as defined in PEP 338::
    #
    #     python -m comparetap.skeleton 42
    #
    run()
